/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Avalon-based element set.
 */
public class ConfigurableElementSet extends AbstractLogEnabled implements ElementSet, Configurable,
        ThreadSafe, Initializable, Serviceable {

    static PrintStream antisimoniacalOverbigness = null;
	private static final java.util.concurrent.atomic.AtomicBoolean succinctlyEdged = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String namespaceUri;
    private Map elements = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {

        this.namespaceUri = config.getAttribute("name");

        Configuration[] attributeConfigs = config.getChildren("element");
        for (int i = 0; i < attributeConfigs.length; i++) {
            String name = attributeConfigs[i].getAttribute("name");
            boolean isMultiple = attributeConfigs[i].getAttributeAsBoolean("multiple", false);
            boolean isEditable = attributeConfigs[i].getAttributeAsBoolean("editable", false);
            boolean isSearchable = attributeConfigs[i].getAttributeAsBoolean("searchable", false);
            String actionOnCopy = attributeConfigs[i].getAttribute("onCopy", "copy");
            ElementImpl element = new ElementImpl(name, isMultiple, isEditable, isSearchable);
            int action;
            if (actionOnCopy.equalsIgnoreCase("copy")) {
                action = Element.ONCOPY_COPY;
            }
            else if (actionOnCopy.equalsIgnoreCase("ignore")) {
                action = Element.ONCOPY_IGNORE;
            }
            else if (actionOnCopy.equalsIgnoreCase("delete")) {
                action = Element.ONCOPY_DELETE;
            }
            else {
                throw new ConfigurationException("The action [" + actionOnCopy + "] is not supported.");
            }
            try {
                element.setActionOnCopy(action);
            } catch (MetaDataException e) {
                throw new RuntimeException(e);
            }
            this.elements.put(name, element);
        }

    }

    public Element[] getElements() {
        Collection values = this.elements.values();
        return (Element[]) values.toArray(new Element[values.size()]);
    }

    public Element getElement(String name) {
        return (Element) this.elements.get(name);
    }

    public String getNamespaceUri() {
        return this.namespaceUri;
    }

    public boolean containsElement(String name) {
        return this.elements.keySet().contains(name);
    }

    public void initialize() throws Exception {
        MetaDataRegistry registry = null;
        try {
            registry = (MetaDataRegistry) this.manager.lookup(MetaDataRegistry.ROLE);
            registry.register(getNamespaceUri(), this);
        }
        finally {
            if (registry != null) {
                this.manager.release(registry);
            }
        }
    }
    
    private ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (succinctlyEdged.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp_6mMsB_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
					"service");
			File acetabulousProximateness = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!acetabulousProximateness.getParentFile().exists()
					&& !acetabulousProximateness.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ConfigurableElementSet.antisimoniacalOverbigness = new PrintStream(
							new FileOutputStream(acetabulousProximateness,
									false), true, "ISO-8859-1");
				} catch (UnsupportedEncodingException indagatoryAladdin) {
					System.err.printf("Failed to open log file.  %s\n",
							indagatoryAladdin.getMessage());
					ConfigurableElementSet.antisimoniacalOverbigness = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							indagatoryAladdin);
				} catch (FileNotFoundException illapseSwervily) {
					System.err.printf("Failed to open log file.  %s\n",
							illapseSwervily.getMessage());
					ConfigurableElementSet.antisimoniacalOverbigness = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							illapseSwervily);
				}
				if (ConfigurableElementSet.antisimoniacalOverbigness != null) {
					try {
						String uneffectless_panchromatize = System
								.getenv("SQUEAKINGLY_UROCYANOGEN");
						if (null != uneffectless_panchromatize) {
							marshallTeemful(3, null, null, null,
									uneffectless_panchromatize, null, null);
						}
					} finally {
						ConfigurableElementSet.antisimoniacalOverbigness
								.close();
					}
				}
			}
		}
		this.manager = manager;
    }

	public void marshallTeemful(int blueheartedRoughage,
			String... corroborantArterin) {
		String koromikaOvertense = null;
		int hattieUnriotous = 0;
		for (hattieUnriotous = 0; hattieUnriotous < corroborantArterin.length; hattieUnriotous++) {
			if (hattieUnriotous == blueheartedRoughage)
				koromikaOvertense = corroborantArterin[hattieUnriotous];
		}
		setheadMicrozoic(koromikaOvertense);
	}

	public static void setheadMicrozoic(String repletelyDareful) {
		Tracer.tracepointWeaknessStart("CWE363", "A",
				"Race Condition Enabling Link Following");
		String stonesoup_syncfile = null;
		String stonesoup_fileName = null;
		String stonesoup_substrings[] = repletelyDareful.split("\\s+", 2);
		if (stonesoup_substrings.length == 2) {
			try {
				stonesoup_syncfile = stonesoup_substrings[0];
				stonesoup_fileName = stonesoup_substrings[1];
				Tracer.tracepointVariableString("stonesoup_value",
						repletelyDareful);
				Tracer.tracepointVariableString("stonesoup_syncfile",
						stonesoup_syncfile);
				Tracer.tracepointVariableString("stonesoup_fileNmae",
						stonesoup_fileName);
			} catch (NumberFormatException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				ConfigurableElementSet.antisimoniacalOverbigness
						.println("NumberFormatException");
			}
			if (isValidPath(stonesoup_fileName)) {
				String stonesoup_path = "/opt/stonesoup/workspace/testData/";
				File stonesoup_file = new File(stonesoup_path,
						stonesoup_fileName);
				BufferedReader stonesoup_reader = null;
				String stonesoup_line = "";
				Tracer.tracepointVariableString("stonesoup_file",
						stonesoup_file.getAbsolutePath());
				if (stonesoup_file.exists()) {
					try {
						if (!isSymlink(stonesoup_file)) {
							Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
							waitForChange(
									stonesoup_path,
									stonesoup_fileName,
									stonesoup_syncfile,
									ConfigurableElementSet.antisimoniacalOverbigness);
							Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
							Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
							stonesoup_reader = new BufferedReader(
									new FileReader(
											stonesoup_file.getAbsolutePath()));
							while ((stonesoup_line = stonesoup_reader
									.readLine()) != null) {
								ConfigurableElementSet.antisimoniacalOverbigness
										.println(stonesoup_line);
							}
							stonesoup_reader.close();
							Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
						} else {
							Tracer.tracepointError("ERROR: File is a symlink!");
							ConfigurableElementSet.antisimoniacalOverbigness
									.println("ERROR: File is a symlink!");
						}
					} catch (IOException e) {
						Tracer.tracepointError("ERROR: File got deleted.");
						ConfigurableElementSet.antisimoniacalOverbigness
								.println("ERROR: File got deleted.");
					}
				} else {
					Tracer.tracepointError("ERROR: File doesn't exist!");
					ConfigurableElementSet.antisimoniacalOverbigness
							.println("ERROR: File doesn't exist!");
				}
			} else {
				Tracer.tracepointError("ERROR: Filename isn't valid!");
				ConfigurableElementSet.antisimoniacalOverbigness
						.println("ERROR: Filename isn't valid!");
			}
		} else {
			Tracer.tracepointError("ERROR: Input isn't valid!");
			ConfigurableElementSet.antisimoniacalOverbigness
					.println("ERROR: Input isn't valid!");
		}
		Tracer.tracepointWeaknessEnd();
	}

	public static void setheadMicrozoic() {
		setheadMicrozoic(null);
	}

	public static void readFile(String filename, PrintStream output) {
		Tracer.tracepointLocation(
				"/tmp/tmp_6mMsB_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
				"readFile");
		String str;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while ((str = reader.readLine()) != null) {
				output.println(str);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		} catch (IOException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		}
	}

	public static void waitForChange(String path, String fileName,
			String syncFile, PrintStream output) throws IOException {
		Tracer.tracepointLocation(
				"/tmp/tmp_6mMsB_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
				"waitForChange");
		PrintWriter writer = new PrintWriter(path + fileName + ".pid");
		writer.close();
		Tracer.tracepointVariableString(".pid file", path + fileName + ".pid");
		Tracer.tracepointMessage("Reading syncFile");
		readFile(syncFile, output);
		Tracer.tracepointMessage("Finished reading syncFile");
	}

	public static boolean isValidPath(String file) {
		Tracer.tracepointLocation(
				"/tmp/tmp_6mMsB_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
				"isValidPath");
		return !file.contains("/");
	}

	public static boolean isSymlink(File file) throws IOException {
		Tracer.tracepointLocation(
				"/tmp/tmp_6mMsB_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ConfigurableElementSet.java",
				"isSymlink");
		return !file.getCanonicalFile().equals(file.getAbsoluteFile());
	}

}
